c     Example code on how to call PES function. This example 
c     prints the results shown in Table I. Stationary points 
c     of H2-C3N- ...    

      program main
      implicit double precision (a-h,o-z)
      dimension rtab(4),t1tab(4),t2tab(4),eref(4),phitab(4)
      character*4 label(4)
      data rtab/9.39d0,8.89d0,6.42d0,6.0d0/
      data t1tab/0d0,0.d0,173.11d0,97.65d0/
      data t2tab/180.0d0,0.0d0,77.81d0,81.14/
      data phitab/0.0d0,0.0d0,0.0d0,0.0d0/
      data eref/-769.75d0,-561.77d0,-348.93d0,-249.52d0/
      data label/'MIN1','MIN2','SP1','SP2'/
      write(6,*)'Stationary points of the H2-C3N- potential ', 
     >           'energy surface'
      write(6,88)
      do i = 1,4
         r=rtab(i)
         theta1=t1tab(i)
         theta2=t2tab(i)
         phi=phitab(i)
         energy=poth2c3nm(r,theta1,theta2,phi)
         write(6,89)label(i),r,theta1,theta2,phi,energy,eref(i)
      enddo
88    format(8x,'R(a0)',2x,'theta1(deg)',2x,'theta2(deg)',2x,
     >       '   phi(deg)',2x,'Energy(cm-1)', 2x,'Energy ref. (cm-1)') 
89    format(a4,3x,f5.2,3(4x,f6.2,4x),3x,2(f7.2,7x))
      end

c----------------------------------------------------------------------c
      double precision function poth2c3nm(r,theta1,theta2,phi)
c----------------------------------------------------------------------c
c  He-C3N- interaction energy (in cm-1) as a function of Jacobi        c
c  coordinates. The units for R are bohr while the angles are in       c
c  degrees. The index 1 refer to H2 and 2 to C3N-                      c
c  Important: the coefficient "COEFF" file must be placed in the same  c
c  directory that executable.                                          c
c  Authors: Miguel Lara-Moreno, Thierry Stoecklin, Philippe Halvick    c
c  Compiler: ifort 13.0.1, gfortran 5.4.0                              c
c----------------------------------------------------------------------c
      implicit double precision (a-h,o-z)
      parameter (beta=6.0d0, a0=1.8d0, r0=14.0d0, nsr=26, l1maxsr=6,
     >           l2maxsr=18, mmaxsr=4, l1maxlr=2, l2maxlr=6, mmaxlr=2,
     >           kmin=3,kmax=8,l1sym=2, l2sym=1,nfbsr=243,nfblr=25,
     >           nfb=432,pi=3.1415926535897932d0)
                       
      dimension fonbas(nfb),angle(3),rvalsr(nsr)
      double precision,  save ::  coeffsr(nsr,nfbsr), y2sr(nsr,nfbsr),
     >                            coefflr(nfblr,kmax)
      integer, save ::  ifirst

      data iprint/0/,yp1/1.0d30/,ypn/1.0d30/
      data rvalsr/ 2.00d0,  2.50d0,  3.00d0,  3.50d0,  4.00d0,
     >             4.50d0,  5.00d0,  5.50d0,  6.00d0,  6.50d0,  7.00d0, 
     >             7.50d0,  8.00d0,  8.50d0,  9.00d0,  9.50d0, 10.00d0,
     >             11.00d0, 12.00d0, 13.00d0, 15.00d0, 20.00d0, 25.00d0,
     >             30.00d0, 40.00d0, 50.00d0/ 

      angle(1)=theta1
      angle(2)=theta2
      angle(3)=phi
      angle=angle*pi/180.0d0

      if (ifirst.ne.1) then
       open(10,file='COEFF',status='old')
       do i =1,nfbsr
        read(10,*)(coeffsr(k,i),k=1,nsr)
        call spline(rvalsr,coeffsr(:,i),nsr,yp1,ypn,y2sr(:,i))
       enddo
       do i =1,nfblr
        read(10,*)(coefflr(i,k),k=kmin,kmax)
       enddo
       close(10)
       ifirst=1
      endif


      n1=nfb
      call primbasis(iprint,l1maxsr,l2maxsr,l1sym,l2sym,mmaxsr,
     >                 angle,nfbsr,fonbas)
      ysr=0.0d0
      do j= 1,nfbsr
        call splint(rvalsr,coeffsr(:,j),y2sr(:,j),nsr,r,coeff_int)
        ysr=ysr+fonbas(j)*coeff_int
      enddo


      call primbasis(iprint,l1maxlr,l2maxlr,l1sym,l2sym,mmaxlr,
     >                 angle,nfblr,fonbas)


        ylr=0.0d0
        do j= 1,nfblr
         do k=kmin,kmax
          ylr=ylr+fonbas(j)*coefflr(j,k)*ttdamp(r,beta,k)/r**k
         enddo
        enddo


        s=0.5d0*(1-tanh(a0*(r-r0)))
        poth2c3nm=s*ysr+(1-s)*ylr
       return
       end

c-------------------------------------------------------------------------
      double precision function ttdamp(gr,beta,kmax)
c-------------------------------------------------------------------------
      implicit double precision(a-h,o-z)

      x = gr*beta
      expox = exp(-x)
      sum = 1.d0
      factk = 1.d0
      do k=1,kmax
        factk = factk * k
        sum = sum + x**k / factk
      enddo
      ttdamp = 1.d0 - expox*sum

      return
  101 format(f12.6,f16.6)
      end

c--------------------------------------------------------------------------
      subroutine splint(xa,ya,y2a,n,x,y)
c--------------------------------------------------------------------------
      integer n
      double precision x,y,xa(n),y2a(n),ya(n)
      integer k,khi,klo
      double precision  a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) pause 'bad xa input in splint'
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**
     *2)/6.
      return
      end

c--------------------------------------------------------------------------
      subroutine spline(x,y,n,yp1,ypn,y2)
c--------------------------------------------------------------------------
      integer n,NMAX
      double precision yp1,ypn,x(n),y(n),y2(n)
      parameter (NMAX=500)
      integer i,k
      double precision p,qn,sig,un,u(NMAX)
      if (yp1.gt..99d30) then
        y2(1)=0.0d0
        u(1)=0.0d0
      else
        y2(1)=-0.5d0
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do  i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.0d0
        y2(i)=(sig-1.0d0)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     *         /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
      enddo
      if (ypn.gt..99d30) then
        qn=0.0
        un=0.0
      else
        qn=0.5d0
        un=(3.0d0/(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.0d0)
      do 12 k=n-1,1,-1
         y2(k)=y2(k)*y2(k+1)+u(k)

12    continue
      return
      end
       
c-----------------------------------------------------------------------
      subroutine primbasis(iprint,l1max,l2max,l1sym,l2sym,mmax,angles,
     >                     nang,fonbas)
c-----------------------------------------------------------------------
      implicit double precision (a-h,o-z)
      dimension angles(3), fonbas(nang)
      allocatable plmt1(:,:), plmt2(:,:)
      logical norm,first
      data norm/.true./,first/.true./
      
      allocate(plmt1(0:l1max,0:l1max),plmt2(0:l2max,0:l2max))
      xt1 = cos(angles(1))
      call flegan(l1max,l1max,norm,xt1,plmt1)
      xt2 = cos(angles(2))
      call flegan(l2max,l2max,norm,xt2,plmt2)
      ifb = 0
      do 10 l1=0,l1max,l1sym
        do 10 l2=0,l2max,l2sym
	  do 15 m=0,min(l1,l2,mmax)
	    ifb = ifb + 1
	    if (ifb.gt.nang) stop 'PRIMBASIS: IFB > NANG'
            if (first.and.iprint.ge.1)
     >          write(6,103) ifb,l1,l2,m,plmt1(l1,m),plmt2(l2,m)
	    fonbas(ifb) = plmt1(l1,m) * plmt2(l2,m) * cos(m*angles(3))
   15     continue
   10 continue
      first = .false.

      return
  103 format(i3,', l1 =',i3,', l2 =',i3,', m =',i3,
     >       ', P1 =',f10.6,', P2 =',f10.6)
      end

c-----------------------------------------------------------------------
      subroutine lnfactrl(n,ftab)
c-----------------------------------------------------------------------
      implicit double precision (a-h,o-z)
      dimension ftab(0:n)

      if (n.lt.0) then
	write(6,101) n
	stop
      end if

      ftab(0) = 0.d0  
      if (n.eq.0) return
      ftab(1) = 0.d0
      do 10 i=2,n
        ftab(i) = dlog(dble(i)) +  ftab(i-1)
   10 continue

      return
  100 format(i3,2e17.6e3)
  101 format(/'Erreur dans FACTRL: N =',i5)
      end

c-----------------------------------------------------------------------
      subroutine flegan(lmax,mmax,norm,x,plm)
c-----------------------------------------------------------------------
      implicit double precision (a-h,o-z)
      dimension plm(0:lmax,0:mmax)
      logical norm
      double precision, allocatable, dimension(:) :: ftab
      data argmax/708.d0/

      if (mmax.gt.lmax) mmax=lmax
      if (abs(x).gt.1.d0) then
	write(6,100) x
	stop
      end if

      sinteta = dsqrt(1.d0-x*x)
      plm(0,0) = 1.d0
      if (lmax.gt.0) then
        do 10 m=1,mmax
   10   plm(m,m) = plm(m-1,m-1) * (2*m-1)*sinteta

        do 20 m=0,min(mmax,lmax-1)
	  plm(m+1,m) = x*(2*m+1)*plm(m,m)  
          do 20 l=m+2,lmax
	  plm(l,m) = (x*(2*l-1)*plm(l-1,m)-(l+m-1)*plm(l-2,m)) / (l-m)
   20   continue
      end if

      if (norm) then
        ifmax = lmax + mmax
        allocate(ftab(0:ifmax))
        call lnfactrl(ifmax,ftab)

        do 30 l=0,lmax
	  fl = dsqrt(dble(2*l+1)/2.d0)
	  do 30 m=0,min(l,mmax)
            argu = 0.5d0*(ftab(l-m)-ftab(l+m))
	    if (dabs(argu).gt.argmax) then
              write(6,102)
              stop
            else
              plm(l,m) = plm(l,m) * fl * dexp(argu)
	    end if
   30   continue
      end if
c
      return
  100 format(/'Erreur dans FLEGAN: X =',f22.16,' est plus grand',
     >       ' que 1.0 en valeur absolue')
  102 format(/'Erreur dans FLEGAN: Underflow ou Overflow dans le ',
     >      ' calcul de l''exponentielle du facteur de normalisation')
      end
